`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Boston University
// Engineer: Benjamin Havey
// 
// Create Date: 03/19/2014 02:00:35 PM
// Design Name: Differential Signal to Single Ended
// Module Name: DStoSE
// Project Name: dDOSI dSAU
// Target Devices: Artix-7 FPGA
// Tool Versions: Vivado 2013.4
// Description: Takes the differential signals from the ADC and converts
//     them to single ended signals. Includes the clock signal and the
//     double data rate portion - using SelectIO intrinsics.
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

/*
module DStoSE(DA0_P,DA0_N,DA2_P,DA2_N,DA4_P,DA4_N,DA6_P,DA6_N,DA8_P,
DA8_N,DA10_P,DA10_N,DA12_P,DA12_N,DB0_P,DB0_N,DB2_P,DB2_N,DB4_P,
DB4_N,DB6_P,DB6_N,DB8_P,DB8_N,DB10_P,DB10_N,DB12_P,DB12_N,
CLK_IN_P, CLK_IN_N, ADC_OUT, CLK_OUT);
*/
/*
module DStoSE(DA0_P,DA0_N,DA2_P,DA2_N,DA4_P,DA4_N,DA6_P,DA6_N,DA8_P,
DA8_N,DA10_P,DA10_N,DA12_P,DA12_N,DB0_P,DB0_N,DB2_P,DB2_N,DB4_P,
DB4_N,DB6_P,DB6_N,DB8_P,DB8_N,DB10_P,DB10_N,DB12_P,DB12_N,
clk0, ADC_OUT, CLK_OUT);
*/
module DStoSE(DA, DB, clk_in1_p, clk_in1_n, ADC_OUT, CLK_OUT);
    //Differential Data Inputs
//    input wire DA0_P,DA0_N,DA2_P,DA2_N,DA4_P,DA4_N,DA6_P,DA6_N,DA8_P,
//        DA8_N,DA10_P,DA10_N,DA12_P,DA12_N;
//    input wire DB0_P,DB0_N,DB2_P,DB2_N,DB4_P,DB4_N,DB6_P,DB6_N,DB8_P,
//        DB8_N,DB10_P,DB10_N,DB12_P,DB12_N;
    input wire clk_in1_p, clk_in1_n;
    //DA is the inputs for port A, DA[0] = DA0_P, DA[1] = DA0_N, DA[2] = DA2_P, etc.
    input wire [13:0] DA;
    //DB is the inputs for port B, same setup as DA.
    input wire [13:0] DB;    

    //Single Ended Data Outputs
    wire DA0,DA2,DA4,DA6,DA8,DA10,DA12;
    wire DB0,DB2,DB4,DB6,DB8,DB10,DB12;

    output [31:0] ADC_OUT;
    output CLK_OUT;
    
//    wire clk1;

    wire clk1;
    assign clk1 = CLK_OUT;
    
    IBUFGDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFGDS_inst (.O(CLK_OUT),.I(clk_in1_n),.IB(clk_in1_p));
    
//    BUFG BUFG_inst (.O(clk1),.I(clk));

    //Single ended Data and Clock outputs.

    //Clock out of the DS->SE clock conversion.

//    assign CLK_OUT = clk;
    assign ADC_OUT[14] = 1'b0;
    assign ADC_OUT[15] = 1'b0;
    assign ADC_OUT[30] = 1'b0;
    assign ADC_OUT[31] = 1'b0;
//    assign ADC_OUT[14] = DB0;
//    assign ADC_OUT[15] = DB2;
//    assign ADC_OUT[30] = DA0;
//    assign ADC_OUT[31] = DA2;
        
    //Input LVDS pairs for Channel A. Need to swap P/N pins because of board layout.
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A0 (.O(DA0),.I(DA[1]),.IB(DA[0]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A2 (.O(DA2),.I(DA[3]),.IB(DA[2]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A4 (.O(DA4),.I(DA[5]),.IB(DA[4]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A6 (.O(DA6),.I(DA[7]),.IB(DA[6]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A8 (.O(DA8),.I(DA[9]),.IB(DA[8]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A10 (.O(DA10),.I(DA[11]),.IB(DA[10]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_A12 (.O(DA12),.I(DA[13]),.IB(DA[12]));

    //Create the DDR single ended data buffers for Channel B
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B0 (.O(DB0),.I(DB[0]),.IB(DB[1]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B2 (.O(DB2),.I(DB[2]),.IB(DB[3]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B4 (.O(DB4),.I(DB[4]),.IB(DB[5]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B6 (.O(DB6),.I(DB[6]),.IB(DB[7]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B8 (.O(DB8),.I(DB[8]),.IB(DB[9]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B10 (.O(DB10),.I(DB[10]),.IB(DB[11]));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B12 (.O(DB12),.I(DB[12]),.IB(DB[13]));
    
    //Convert DDR to two SDR channels, put output on ADC_OUTA
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a0 (.Q1(ADC_OUT[16]),.Q2(ADC_OUT[17]),.D(!DA0),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a2 (.Q1(ADC_OUT[18]),.Q2(ADC_OUT[19]),.D(!DA2),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a4 (.Q1(ADC_OUT[20]),.Q2(ADC_OUT[21]),.D(!DA4),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a6 (.Q1(ADC_OUT[22]),.Q2(ADC_OUT[23]),.D(!DA6),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a8 (.Q1(ADC_OUT[24]),.Q2(ADC_OUT[25]),.D(!DA8),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a10 (.Q1(ADC_OUT[26]),.Q2(ADC_OUT[27]),.D(!DA10),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a12 (.Q1(ADC_OUT[28]),.Q2(ADC_OUT[29]),.D(!DA12),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    
    //Convert DDR to two SDR channels, put output on ADC_OUTB
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b0 (.Q1(ADC_OUT[0]),.Q2(ADC_OUT[1]),.D(DB0),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b2 (.Q1(ADC_OUT[2]),.Q2(ADC_OUT[3]),.D(DB2),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b4 (.Q1(ADC_OUT[4]),.Q2(ADC_OUT[5]),.D(DB4),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b6 (.Q1(ADC_OUT[6]),.Q2(ADC_OUT[7]),.D(DB6),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b8 (.Q1(ADC_OUT[8]),.Q2(ADC_OUT[9]),.D(DB8),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b10 (.Q1(ADC_OUT[10]),.Q2(ADC_OUT[11]),.D(DB10),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b12 (.Q1(ADC_OUT[12]),.Q2(ADC_OUT[13]),.D(DB12),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
/*
    //Create the DDR single ended data buffers for Channel B
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B0 (.O(DB0),.I(DB0_P),.IB(DB0_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B2 (.O(DB2),.I(DB2_P),.IB(DB2_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B4 (.O(DB4),.I(DB4_P),.IB(DB4_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B6 (.O(DB6),.I(DB6_P),.IB(DB6_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B8 (.O(DB8),.I(DB8_P),.IB(DB8_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B10 (.O(DB10),.I(DB10_P),.IB(DB10_N));
    IBUFDS #(.DIFF_TERM("TRUE"), .IBUF_LOW_PWR("FALSE"), .IOSTANDARD("25LVDS"))
        IBUFDS_B12 (.O(DB12),.I(DB12_P),.IB(DB12_N));
        */
    /*
    //Convert DDR to two SDR channels, put output on ADC_OUTA
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a0 (.Q1(ADC_OUT[16]),.Q2(ADC_OUT[17]),.D(!DA0),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a2 (.Q1(ADC_OUT[18]),.Q2(ADC_OUT[19]),.D(!DA2),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a4 (.Q1(ADC_OUT[20]),.Q2(ADC_OUT[21]),.D(!DA4),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a6 (.Q1(ADC_OUT[22]),.Q2(ADC_OUT[23]),.D(!DA6),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a8 (.Q1(ADC_OUT[24]),.Q2(ADC_OUT[25]),.D(!DA8),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a10 (.Q1(ADC_OUT[26]),.Q2(ADC_OUT[27]),.D(!DA10),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_a12 (.Q1(ADC_OUT[28]),.Q2(ADC_OUT[29]),.D(!DA12),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    
    //Convert DDR to two SDR channels, put output on ADC_OUTB
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b0 (.Q1(ADC_OUT[0]),.Q2(ADC_OUT[1]),.D(DB0),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b2 (.Q1(ADC_OUT[2]),.Q2(ADC_OUT[3]),.D(DB2),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b4 (.Q1(ADC_OUT[4]),.Q2(ADC_OUT[5]),.D(DB4),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b6 (.Q1(ADC_OUT[6]),.Q2(ADC_OUT[7]),.D(DB6),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b8 (.Q1(ADC_OUT[8]),.Q2(ADC_OUT[9]),.D(DB8),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b10 (.Q1(ADC_OUT[10]),.Q2(ADC_OUT[11]),.D(DB10),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
    IDDR #(.DDR_CLK_EDGE("OPPOSITE_EDGE"),.INIT_Q1(1'b0),.INIT_Q2(1'b0),.SRTYPE("SYNC"))
        IDDR_b12 (.Q1(ADC_OUT[12]),.Q2(ADC_OUT[13]),.D(DB12),.C(clk1),.CE(1'b1),.S(1'b0),.R(1'b0));
*/
endmodule
